name = "climextremes"

import pkg_resources  # part of setuptools
__version__ = pkg_resources.require("climextremes")[0].version

def reinstall_climextremes(force, version):
  import os
  import sys
  import rpy2.robjects
  import warnings
  from rpy2.rinterface import RRuntimeWarning
  warnings.filterwarnings("ignore", category=RRuntimeWarning)

  def install_cran_default(repos = 'https://cran.us.r-project.org'):
    # sometimes failure occurs with URL issue with cran.r-project.org, so try a mirror
    try:     # this may fail with error or fail but only issue a warning
      rpy2.robjects.r("""install.packages('climextRemes',repos='{0}')""".format(repos))
      rpy2.robjects.r('''library(climextRemes)''')
    except:  
      rpy2.robjects.r("""install.packages('climextRemes',repos='https://cran.cnr.berkeley.edu')""")
      rpy2.robjects.r('''library(climextRemes)''')

  def get_devtools(repos = 'https://cran.us.r-project.org'):
    try:
      rpy2.robjects.r('''library(devtools)''')
      return True
    except:
      try:
        rpy2.robjects.r("""install.packages('devtools',repos='{0}')""".format(repos))
      except:
        rpy2.robjects.r("""install.packages('devtools',repos='https://cran.cnr.berkeley.edu')""")
    try:
      rpy2.robjects.r('''library(devtools)''')
    except:
      print("Unable to install R package devtools; needed for version-specific installation of R climextRemes.")
      return False
    return True
        

  def install_cran_specific(version, repos = 'https://cran.us.r-project.org'):
    check = get_devtools()
    if check:
      try:
        rpy2.robjects.r("""install_version('climextRemes','{0}',repos='{1}')""".format(version, repos))
        rpy2.robjects.r('''library(climextRemes)''')
      except:
        rpy2.robjects.r("""install_version('climextRemes','{0}',repos='https://cran.cnr.berkeley.edu')""".format(version, repos))
        rpy2.robjects.r('''library(climextRemes)''')
  
  if force:
    try:
      if version is None:
        install_cran_default()
      else:
        try:
          install_cran_specific(version)
        except:
          print("Installation of R climextRemes version: " + version + " failed, likely because the version is not available on the CRAN R package archive.")
          return False
    except:
      return False
  else:
    try:
      test_import = rpy2.robjects.r('''library(climextRemes)''')
      iv = rpy2.robjects.r('''packageVersion("climextRemes")''')[0]
      installed_version = str(iv[0]) + "." + str(iv[1]) + "." + str(iv[2])

      if version is not None and version != installed_version:
        print("Current version: "  + installed_version + " does not match requested version: " + version + ". Attempting installation of R climextRemes package (this may take a few minutes) ...")
        try:
          install_cran_specific(version)
        except:
          print("Installation of R climextRemes version: " + version + " failed (likely because the version is not on the CRAN R package archive).\nFalling back to installed version.")
    except:
      # initial import failed
      print("Attempting installation of R climextRemes package and its dependencies (this may take a few minutes) ...")
      try:
        if version is None:
          install_cran_default()
        else:
          try:
            install_cran_specific(version)
          except:
            print("Installation of version: " + version + " failed (likely because the version is not on the CRAN R package archive).\nFalling back to installing default CRAN package.")
            install_cran_default()
      except:
        return False
  return True

def __wrap_import():
  import os
  import sys
  import rpy2.robjects

  global __version__

  if not reinstall_climextremes(False, __version__):
      print("Installation of climextRemes failed. Please manually install climextRemes using CRAN R package archive.")
      return

  # Force R warnings to come through to Python
  rpy2.robjects.r('options(warn=1)')
  import warnings
  from rpy2.rinterface import RRuntimeWarning
  warnings.filterwarnings("always", category=RRuntimeWarning)

  __climextRemes_home__ = rpy2.robjects.r('''
  library(climextRemes)
  sp <- searchpaths()
  climextremes_path <- sp[grep("climextRemes", sp)]
  ''')[0]
  __climextRemes_python_path__ = __climextRemes_home__ + "/python"
  sys.path.append(__climextRemes_python_path__)

def __cleanup_import():
  import sys
  del sys.path[-1]
  del sys.path[-1]

__wrap_import()
from climextRemes_wrapper import *
__cleanup_import()




"""
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
# with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
long_description = """
Functions for fitting GEV and POT (via point process fitting)
models for extremes in climate data, providing return values, return
probabilities, and return periods for stationary and nonstationary models.
Also provides differences in return values and differences in log return
probabilities for contrasts of covariate values. Functions for estimating risk
ratios for event attribution analyses, including uncertainty. Under the hood,
many of the functions use functions from 'extRemes', including for fitting the
statistical models.
"""

version = {}
with open('version.py') as f:
    exec(f.read(), version)
    
    
setup(
    name='climextremes',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version['__version__'], 

    description='climextremes wrapper module',
    long_description=long_description,

    # The project's main homepage.
    url='https://bitbucket.org/lbl-cascade',

    # Author details
    author='Christopher Paciorek',
    author_email='paciorek@berkeley.edu',

    # Choose your license
    license='BSD License',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        #'Development Status :: 3 - Alpha',
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Science/Research',
        #'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        #'License :: OSI Approved :: MIT License',
        'License :: OSI Approved :: BSD License',
        
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
	'Programming Language :: Python :: 3.8',
    ],

    # What does your project relate to?
    keywords=['statistics', 'GEV', 'POT', 'detection and attribution'],
    packages=['climextremes'],
    install_requires=['numpy', 'pandas', 'rpy2', 'tzlocal'],
)
